<?xml version="1.0"?>
<!--
***********************************************************************
*
*  Source File Name = nucleotide1_0.xsl
*
*  (C) COPYRIGHT International Business Machines Corp. 2003,2004 
*  All Rights Reserved
*  Licensed Materials - Property of IBM
*
*  US Government Users Restricted Rights - Use, duplication or
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*
*  Function = XSL Stylesheet for the Nucleotide schema
*
*  Operating System = All
*
***********************************************************************
-->

<xsl:stylesheet id="nucleotide1_0"
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="xml" indent="yes"/>

<!-- Generate the sequence join key                                  -->
<xsl:template match="GBSeq">
  <xsl:element name="{name()}">
    <xsl:element name="SequenceJoinKey">
      <xsl:value-of select="generate-id()"/>
    </xsl:element>
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates select="*|text()"/>
  </xsl:element>
</xsl:template>


<!-- Normal dates                                 -->
<xsl:template match="GBSeq_create-date|GBSeq_update-date">
  <xsl:call-template name="normal-date"/>
</xsl:template>

<!-- Handle the most common forms of semicolon separated list       -->
<xsl:template match="GBSeq_keywords|GBReference_authors">
  <xsl:call-template name="semicolon-list">
    <xsl:with-param name="listName"><xsl:value-of select="name()"/></xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!-- Extract version number from GBSeq_accesion-version             -->
<xsl:template match="GBSeq_accession-version">
  <GBSeq_version>
    <xsl:value-of select="substring-after(text(), '.')"/>
  </GBSeq_version>
</xsl:template>

<!-- Extract GI number from GBSeq_other-seqid                       -->
<xsl:template match="GBSeq_other-seqids/GBSeqid[substring(text(), 1, 2) = 'gi']">
  <GBSeq_gi>
    <xsl:value-of select="substring-after(text(), '|')"/>
  </GBSeq_gi>
</xsl:template>

<!-- Break GBReference_reference into number and range              -->
<xsl:template match="GBReference_reference">
  <xsl:element name="GBReference_reference">
    <xsl:element name="ReferenceNum">
      <xsl:value-of select="substring-before(text(), ' (')"/>
    </xsl:element>
    <xsl:element name="RangeLow">
      <xsl:value-of select="substring-before(substring-after(text(), 'bases '), ' ')"/>
    </xsl:element>
    <xsl:element name="RangeHigh">
      <xsl:value-of select="substring-before(substring-after(text(), 'to '), ')')"/>
    </xsl:element>
  </xsl:element>
</xsl:template>

<!-- Generate the feature join key                                  -->
<xsl:template match="GBFeature">
  <xsl:element name="{name()}">
    <xsl:element name="FeatureJoinKey">
      <xsl:value-of select="generate-id()"/>
    </xsl:element>
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates select="*|text()"/>
  </xsl:element>
</xsl:template>

<!-- Reject Templates                                               -->
<!-- These templates prevent the "catchall" (see below) template    -->
<!-- from copying elements that we don't care about; this should    -->
<!-- reduce the amount of work that needs to be done by the next    -->
<!-- phase.                                                         -->
<xsl:template match="Seqid"/>

<!-- Utility Templates                                              -->

<!-- "normal" dates                                                 -->
<!-- These are converted to a XML schema format date (yyyy-mm-dd)   -->
<!-- from DD-Mon-YYYY                                               -->
<xsl:template name="normal-date">
  <xsl:variable name='tmonth'>
    <xsl:value-of select="translate(substring(text(),4,3),'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:element name="{name()}">
    <xsl:value-of select="substring(text(), 8,4)" />
    <xsl:choose>
      <xsl:when test="$tmonth = 'JAN'">-01-</xsl:when>
      <xsl:when test="$tmonth = 'FEB'">-02-</xsl:when>
      <xsl:when test="$tmonth = 'MAR'">-03-</xsl:when>
      <xsl:when test="$tmonth = 'APR'">-04-</xsl:when>
      <xsl:when test="$tmonth = 'MAY'">-05-</xsl:when>
      <xsl:when test="$tmonth = 'JUN'">-06-</xsl:when>
      <xsl:when test="$tmonth = 'JUL'">-07-</xsl:when>
      <xsl:when test="$tmonth = 'AUG'">-08-</xsl:when>
      <xsl:when test="$tmonth = 'SEP'">-09-</xsl:when>
      <xsl:when test="$tmonth = 'OCT'">-10-</xsl:when>
      <xsl:when test="$tmonth = 'NOV'">-11-</xsl:when>
      <xsl:when test="$tmonth = 'DEC'">-12-</xsl:when>
    </xsl:choose>
    <xsl:value-of select="substring(text(), 1,2)" />
    <!-- <xsl:value-of select="concat('/', substring(text(), 1,2), '/', substring(text(), 8,4))"/> -->
  </xsl:element>
</xsl:template>

<xsl:template name="semicolon-list">
  <xsl:param name="listName"/>
  <xsl:element name="{$listName}">
    <xsl:for-each select="child::*">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:value-of select="."/>
    </xsl:for-each>
  </xsl:element>
</xsl:template>

<!-- "Catchall" template                                            -->
<!-- This template causes any element, not otherwise processed      -->
<!-- to be copied to the output                                     -->
<xsl:template match="*">
  <xsl:element name="{name()}">
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates select="*|text()"/>
  </xsl:element>
</xsl:template>

<xsl:template match="@*">
  <xsl:attribute name="{name()}">
    <xsl:value-of select="."/>
  </xsl:attribute>
</xsl:template>

</xsl:stylesheet>