<?xml version="1.0"?>
<!--
***********************************************************************
*
*  Source File Name = omim1_0.xsl
*
*  (C) COPYRIGHT International Business Machines Corp. 2003,2004 
*  All Rights Reserved
*  Licensed Materials - Property of IBM
*
*  US Government Users Restricted Rights - Use, duplication or
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*
*  Function = XSL Stylesheet for the Omim schema
*
*  Operating System = All
*
***********************************************************************
-->

<xsl:stylesheet id="omim1_0"
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="xml" indent="yes"/>


<!-- ============================================================== -->
<!-- ============================================================== -->
<!-- OMIM templates                                                 -->
<!-- The templates below implement changes required by the OMIM     -->
<!-- nickname schema/layout                                         -->
<!-- ============================================================== -->
<!-- ============================================================== -->

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Modify dates to DB2 format                                     -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template match="Mim-date">
  <xsl:call-template name="db2_date"/>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Modify Mim-entry_mimType display                               -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template match="Mim-entry_mimType">
  <xsl:element name="Mim-entry_mimNumberDisplay_db2">
    <xsl:choose>
      <xsl:when test="@value = 'plus'">+</xsl:when>
      <xsl:when test="@value = 'pound'">#</xsl:when>
      <xsl:when test="@value = 'star'">*</xsl:when>
      <xsl:when test="@value = 'perc'">%</xsl:when>
      <xsl:when test="@value = 'none'"></xsl:when>
    </xsl:choose>
    <xsl:value-of select="ancestor::Mim-entry/Mim-entry_mimNumber/text()"/>
  </xsl:element>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Replace 'false'/'true' values by 'N'/'Y'                       -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template match="Mim-entry_hasSummary/@value|Mim-entry_hasSynopsis/@value|Mim-reference_ambiguous/@value|Mim-reference_noLink/@value">
  <xsl:call-template name="db2_boolean"/>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Generate join key elements for child nicknames                 -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template match="Mim-entry_textfields/Mim-text|Mim-entry_allelicVariants/Mim-allelic-variant|Mim-allelic-variant_description/Mim-text">
  <xsl:element name="{name()}">
    <xsl:value-of select="./text()"/>
    <xsl:call-template name="add_join_key"/>
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates select="*|text()"/>
  </xsl:element>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Split string of Mim-link_uids into sequence of elements        -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template match="Mim-link/Mim-link_uids">
  <xsl:element name="{name()}">
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates select="*|text()"/>
  </xsl:element>

  <xsl:call-template name="split_string">
    <xsl:with-param name="string">
      <xsl:value-of select="."/>
    </xsl:with-param>
  </xsl:call-template> 
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Replace sequence of <Mim-author> elements by single element    -->
<!-- that contains semi-colon separated list of authors             -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template match="Mim-reference_authors">
  <xsl:call-template name="semicolon_list">
    <xsl:with-param name="list_name"><xsl:value-of select="name()"/></xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Remove <Mim-entry_medlineLinks>, <Mim-entry_proteinLinks>, ... -->
<!-- elements                                                       -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  --> 
<xsl:template match="Mim-entry_medlineLinks|Mim-entry_proteinLinks|Mim-entry_nucleotideLinks|Mim-entry_structureLinks">
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Add <Mim-entry_omimLinks_db2> element to replace               -->
<!-- <Mim-entry_medlineLinks>, <Mim-entry_proteinLinks>, ...        -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template match="Mim-entry">
  <xsl:element name="Mim-entry">
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates select="*|text()"/>

    <xsl:value-of select="./text()"/>
    <xsl:element name="Mim-entry_omimLinks_db2">
      <xsl:variable name="medlineLinks">
        <xsl:call-template name="extract_elements">
          <xsl:with-param name="element" select="Mim-entry_medlineLinks"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="proteinLinks">
        <xsl:call-template name="extract_elements">
          <xsl:with-param name="element" select="Mim-entry_proteinLinks"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="nucleotideLinks">
        <xsl:call-template name="extract_elements">
          <xsl:with-param name="element" select="Mim-entry_nucleotideLinks"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="structureLinks">
        <xsl:call-template name="extract_elements">
          <xsl:with-param name="element" select="Mim-entry_structureLinks"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:copy-of select="$medlineLinks"/>
      <xsl:copy-of select="$proteinLinks"/>
      <xsl:copy-of select="$nucleotideLinks"/>
      <xsl:copy-of select="$structureLinks"/>
      <xsl:value-of select="./text()"/>
    </xsl:element>
  </xsl:element>

</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Create new edit action element <Mim-edit-item_action_db2>      -->
<!-- using information from <Mim-edit-item_author>                  -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template match="Mim-edit-item">
  <xsl:element name="Mim-edit-item">
    <xsl:value-of select="./text()"/>
    <xsl:element name="Mim-edit-item_action_db2">
      <xsl:value-of select="substring-after(Mim-edit-item_author/text(), ' - ')"/>
    </xsl:element>

    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates select="*|text()"/>
  </xsl:element>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Remove edit action from the <Mim-edit-item_author> element     -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template match="Mim-edit-item_author">
  <xsl:element name="Mim-edit-item_author_db2">
    <xsl:value-of select="substring-before(./text(), ' - ')"/>
  </xsl:element>
</xsl:template>


<!-- ============================================================== -->
<!-- ============================================================== -->
<!-- Utility templates                                              -->
<!-- ============================================================== -->
<!-- ============================================================== -->

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Name: db2_date                                                 -->
<!-- Function: Convert dates to a XML schema format (yyyy-mm-dd)    -->
<!-- Parameter: None                                                -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template name="db2_date">
  <xsl:element name="{concat(name(), '_db2')}">
    <xsl:value-of select = "Mim-date_year/text()"/>-<xsl:choose>
      <xsl:when test="string-length(Mim-date_month/text()) = 1">0<xsl:value-of select="Mim-date_month/text()" /></xsl:when>
      <xsl:otherwise><xsl:value-of select="Mim-date_month/text()" /></xsl:otherwise>
    </xsl:choose>-<xsl:choose>
      <xsl:when test="string-length(Mim-date_day/text()) = 1">0<xsl:value-of select="Mim-date_day/text()" /></xsl:when>
      <xsl:otherwise><xsl:value-of select="Mim-date_day/text()" /></xsl:otherwise>
    </xsl:choose>
    <!-- <xsl:value-of select="concat(Mim-date_month/text(), '/', Mim-date_day/text(), '/', Mim-date_year/text())"/> -->
  </xsl:element>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Name: db2_boolean                                              -->
<!-- Function: Convert 'false' into 'N' and 'true' into 'Y'         -->
<!-- Parameter: None                                                -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template name="db2_boolean">
  <xsl:attribute name="{concat(name(), '_db2')}">
    <xsl:choose>
      <xsl:when test=". = 'false'">N</xsl:when>
      <xsl:when test=". = 'true'">Y</xsl:when>
      <!-- In case upper case is used... -->
      <xsl:when test=". = 'FALSE'">N</xsl:when>
      <xsl:when test=". = 'TRUE'">Y</xsl:when>         
     </xsl:choose>
  </xsl:attribute>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Name: add_join_key                                             -->
<!-- Function: Add a join key element containing a random value     -->
<!-- Parameter: $element_name, the name of the new join key element -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template name="add_join_key">
  <xsl:param name="element_name" select="name()"/>
  <xsl:element name="{concat($element_name, '_joinKey_db2')}">
    <xsl:value-of select="generate-id()"/>
  </xsl:element>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Name: split_string                                             -->
<!-- Function: Invoke split_string_impl template                    -->
<!-- Parameter: $string, the coma separated list                    -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template name="split_string">
  <xsl:param name="string"/>
  <xsl:variable name="result">
    <xsl:call-template name="split_string_impl">
      <xsl:with-param name="all">
        <xsl:value-of select="normalize-space($string)"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:copy-of select="$result"/>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Name: split_string_impl                                        -->
<!-- Function: Split string into sequence of elements               -->
<!-- Parameter: $all, the normalized coma separated list            -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template name="split_string_impl">
  <xsl:param name="all"/>
  <xsl:variable name="first" select="substring-before($all, ',')"/>

  <xsl:choose>
    <xsl:when test="string-length($first)">
      <xsl:value-of select="ancestor::Mim-link/text()"/>
      <Mim-link_uid_db2>
        <xsl:value-of select="$first"/>
      </Mim-link_uid_db2>
      <xsl:call-template name="split_string_impl">
        <xsl:with-param name="all">
            <xsl:value-of select="substring-after($all, ',')"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="ancestor::Mim-link/text()"/>
      <Mim-link_uid_db2>
        <xsl:value-of select="$all"/>
      </Mim-link_uid_db2>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Name: semicolon_list                                           -->
<!-- Function: Replace sequence of elements with one element that   -->
<!--           contains a coma separated list                       -->
<!-- Param: $list_name, the name of the list element                -->
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template name="semicolon_list">
  <xsl:param name="list_name"/>
  <xsl:element name="{concat($list_name, '_db2')}">
    <xsl:for-each select="Mim-author/Mim-author_name">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:value-of select="."/>
    </xsl:for-each>
  </xsl:element>
</xsl:template>

<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!-- Name: extract_elements                                         -->
<!-- Function: Extract child elements for a given element           -->
<!-- Param: $element, the element we want to extract the child      -->
<!--        elements from                                           --> 
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<xsl:template name="extract_elements">
  <xsl:param name="element"/>

  <xsl:if test="$element">
    <xsl:variable name="links">
      <xsl:value-of select="$element/text()"/>
      <xsl:element name="Mim-link">
        <xsl:value-of select="$element/Mim-link/text()"/>
        <xsl:call-template name="add_join_key">
          <xsl:with-param name="element_name">Mim-link</xsl:with-param>
        </xsl:call-template>

        <xsl:value-of select="$element/Mim-link/text()"/>
        <xsl:element name="Mim-link_type_db2"> 
          <xsl:choose>
            <xsl:when test="name($element) = 'Mim-entry_medlineLinks'">Medline</xsl:when>
            <xsl:when test="name($element) = 'Mim-entry_proteinLinks'">Protein</xsl:when>
            <xsl:when test="name($element) = 'Mim-entry_nucleotideLinks'">Nucleotide</xsl:when>
            <xsl:when test="name($element) = 'Mim-entry_structureLinks'">Structure</xsl:when>
          </xsl:choose>
        </xsl:element>
       
        <xsl:value-of select="$element/Mim-link/text()"/>
        <xsl:copy-of select="$element/Mim-link/Mim-link_num"/>

        <xsl:value-of select="$element/Mim-link/text()"/>
        <xsl:apply-templates select="$element/Mim-link/Mim-link_uids"/>
        <xsl:value-of select="$element/text()"/>
      </xsl:element>
    </xsl:variable>

    <xsl:copy-of select="$links"/>
  </xsl:if>  

</xsl:template>


<!-- ============================================================== -->
<!-- ============================================================== -->
<!-- "Catchall" template                                            -->
<!-- These templates cause any element, not otherwise processed     -->
<!-- to be copied to the output                                     -->
<!-- ============================================================== -->
<!-- ============================================================== -->

<xsl:template match="*">
  <xsl:element name="{name()}">
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates select="*|text()"/>
  </xsl:element>
</xsl:template>

<xsl:template match="@*">
  <xsl:attribute name="{name()}">
    <xsl:value-of select="."/>
  </xsl:attribute>
</xsl:template>

</xsl:stylesheet>