<?xml version="1.0"?>
<!--
***********************************************************************
*
*  Source File Name = pubmed1_0.xsl
*
*  (C) COPYRIGHT International Business Machines Corp. 2003,2004 
*  All Rights Reserved
*  Licensed Materials - Property of IBM
*
*  US Government Users Restricted Rights - Use, duplication or
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*
*  Function = XSL Stylesheet for the Pubmed schema
*
*  Operating System = All
*
***********************************************************************
-->

<xsl:stylesheet id="pubmed1_0"
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="xml" indent="yes"/>

<!-- Normal (non-publication) dates                                 -->
<xsl:template match="DateCreated|DateCompleted|DateRevised|DateOfElectronicPublication|ArticleDate">
  <xsl:call-template name="normal-date"/>
</xsl:template>

<!-- Handle the most common forms of semicolon separated list       -->
<xsl:template match="PublicationTypeList|GeneSymbolList|KeywordList">
  <xsl:call-template name="semicolon-list">
    <xsl:with-param name="listName"><xsl:value-of select="name()"/></xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!-- Pagination                                                     -->
<!-- May be one of the following:                                   -->
<!--   1)  MedlinePgn                                               -->
<!--   2)  StartPage                                                -->
<!--   3)  StartPage - EndPage                                      -->
<!--   4)  StartPage - EndPage (MedlinePgn)                         -->
<!--   5)  StartPage (MedlinePgn)                                   -->
<xsl:template match="Pagination">
<Pagination>
  <xsl:choose>
    <xsl:when test="StartPage and EndPage and MedlinePgn">
     <xsl:value-of select="concat(StartPage, '-', EndPage, '(', MedlinePgn, ')')"/>
	</xsl:when>
    <xsl:when test="StartPage and EndPage">
     <xsl:value-of select="concat(StartPage, '-', EndPage)"/>
	</xsl:when>
   <xsl:when test="StartPage and MedlinePgn">
     <xsl:value-of select="concat(StartPage, '(', MedlinePgn, ')')"/>
   </xsl:when>
   <xsl:when test="StartPage">
     <xsl:value-of select="StartPage"/>
   </xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="MedlinePgn"/>
	</xsl:otherwise>
  </xsl:choose>
</Pagination>
</xsl:template>

<!-- Various lists of names                                         -->
<!-- AuthorList, PersonalNameSubjectList, InvestigatorList          -->
<!--   All of these use the "canonical-name" template to put the    -->
<!--   names into a standard form                                   -->
<xsl:template match="AuthorList">
  <AuthorList>
    <xsl:for-each select="Author">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:call-template name="canonical-name"/>
    </xsl:for-each>
  </AuthorList>
</xsl:template>

<xsl:template match="PersonalNameSubjectList">
  <PersonalNameSubjectList>
    <xsl:for-each select="PersonalNameSubject">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:call-template name="canonical-name"/>
    </xsl:for-each>
  </PersonalNameSubjectList>
</xsl:template>

<xsl:template match="InvestigatorList">
  <InvestigatorList>
    <xsl:for-each select="Investigator">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:call-template name="canonical-name"/>
    </xsl:for-each>
  </InvestigatorList>
</xsl:template>


<!-- Registry numbers                                               -->
<!--   We reduce both RegistryNumber and CASRegistryNumber          -->
<!--   to a pair of elements, <RegistryNumber> and <CASRegistry>    -->
<!--   The latter has a value of 'Y' or 'N' depending on the source -->
<!--   of the other value.                                          -->
<xsl:template match="RegistryNumber|CASRegistryNumber">
  <xsl:element name="RegistryNumber">
    <xsl:attribute name="CASRegistry">
      <xsl:choose>
        <xsl:when test="CASRegistryNumber">Y</xsl:when>
        <xsl:otherwise>N</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <xsl:value-of select="text()"/>
  </xsl:element>
</xsl:template>


<!--  The following don't work correctly!  -->
<xsl:template match="Language">
  <LanguageList>
    <xsl:for-each select=".|following-sibling::Language">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:value-of select="."/>
    </xsl:for-each>
  </LanguageList>
</xsl:template>

<xsl:template match="SpaceflightMission">
  <SpaceflightMissionList>
    <xsl:for-each select=".|following-sibling::SpaceflightMission">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:value-of select="."/>
    </xsl:for-each>
  </SpaceflightMissionList>
</xsl:template>

<xsl:template match="CitationSubset">
  <CitationSubsetList>
    <xsl:for-each select=".|following-sibling::CitationSubset">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:value-of select="."/>
    </xsl:for-each>
  </CitationSubsetList>
</xsl:template>


<!-- Publication date                                               -->
<!-- This may be in one of the following forms:                     -->
<!--   1) Year                                                      -->
<!--   2) Year Month                                                -->
<!--   3) Year Month Day                                            -->
<!--   4) Year Season                                               -->
<!--   5) Medline date (unknown syntax)                             -->
<!-- We generate two elements, <PubDate> and <PubDateString>        -->
<!-- The latter contains a canonical string representation of the   -->
<!-- date.  The former contains a DB2-format date, if we can        -->
<!-- derive one from the information provided.  For instance, if we -->
<!-- have a month and no day, then the day is assumed to be '1'.    -->
<!-- TODO: Seasons need to be determined                            -->
<xsl:template match="PubDate">
  <!-- Try to generate a usable date -->
  <PubDate>
    <xsl:choose>
      <xsl:when test="Month and Day">
        <xsl:value-of select="concat(Year/text(), '-')"/>
        <xsl:choose>
          <xsl:when test="Month/text() = 'Jan'">01-</xsl:when>
          <xsl:when test="Month/text() = 'Feb'">02-</xsl:when>
          <xsl:when test="Month/text() = 'Mar'">03-</xsl:when>
          <xsl:when test="Month/text() = 'Apr'">04-</xsl:when>
          <xsl:when test="Month/text() = 'May'">05-</xsl:when>
          <xsl:when test="Month/text() = 'Jun'">06-</xsl:when>
          <xsl:when test="Month/text() = 'Jul'">07-</xsl:when>
          <xsl:when test="Month/text() = 'Aug'">08-</xsl:when>
          <xsl:when test="Month/text() = 'Sep'">09-</xsl:when>
          <xsl:when test="Month/text() = 'Oct'">10-</xsl:when>
          <xsl:when test="Month/text() = 'Nov'">11-</xsl:when>
          <xsl:when test="Month/text() = 'Dec'">12-</xsl:when>
          <xsl:otherwise>01-</xsl:otherwise>
        </xsl:choose>
        <xsl:choose>
          <xsl:when test="Day/text() = '1'">01</xsl:when>
          <xsl:when test="Day/text() = '2'">02</xsl:when>
          <xsl:when test="Day/text() = '3'">03</xsl:when>
          <xsl:when test="Day/text() = '4'">04</xsl:when>
          <xsl:when test="Day/text() = '5'">05</xsl:when>
          <xsl:when test="Day/text() = '6'">06</xsl:when>
          <xsl:when test="Day/text() = '7'">07</xsl:when>
          <xsl:when test="Day/text() = '8'">08</xsl:when>
          <xsl:when test="Day/text() = '9'">09</xsl:when>
          <xsl:otherwise><xsl:value-of select="Day/text()"/></xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="Month">
        <xsl:value-of select="Year/text()"/>
        <xsl:choose>
          <xsl:when test="Month/text() = 'Jan'">-01-01</xsl:when>
          <xsl:when test="Month/text() = 'Feb'">-02-01</xsl:when>
          <xsl:when test="Month/text() = 'Mar'">-03-01</xsl:when>
          <xsl:when test="Month/text() = 'Apr'">-04-01</xsl:when>
          <xsl:when test="Month/text() = 'May'">-05-01</xsl:when>
          <xsl:when test="Month/text() = 'Jun'">-06-01</xsl:when>
          <xsl:when test="Month/text() = 'Jul'">-07-01</xsl:when>
          <xsl:when test="Month/text() = 'Aug'">-08-01</xsl:when>
          <xsl:when test="Month/text() = 'Sep'">-09-01</xsl:when>
          <xsl:when test="Month/text() = 'Oct'">-10-01</xsl:when>
          <xsl:when test="Month/text() = 'Nov'">-11-01</xsl:when>
          <xsl:when test="Month/text() = 'Dec'">-12-01</xsl:when>
          <xsl:otherwise>-01-01</xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="Year">
        <xsl:value-of select="concat(Year/text(), '-01-01')"/>
      </xsl:when>
      <xsl:when test="Season">
  <!-- TODO!!!!!                   -->
      </xsl:when>
    </xsl:choose>
  </PubDate>
  <!-- Generate a string representation -->
  <PubDateString>
  <xsl:choose>
    <xsl:when test="Month and Day">
      <xsl:value-of select="concat(Day/text(), ' ', Month/text(), ' ', Year/text())"/>
    </xsl:when>
    <xsl:when test="Month">
      <xsl:value-of select="concat(Month/text(), ' ', Year/text())"/>
    </xsl:when>
    <xsl:when test="Season">
      <xsl:value-of select="concat(Season/text(), ' ', Year/text())"/>
    </xsl:when>
    <xsl:when test="MedlineDate">
      <xsl:value-of select="MedlineDate/text()"/>
    </xsl:when>
  </xsl:choose>
  </PubDateString>
</xsl:template>

<!-- MeSH Descriptors                                               -->
<!-- Generate an elements, <DescriptorOrName> with an attribute,    -->
<!-- MajorTopicYN                                                   -->
<xsl:template match="Descriptor|DescriptorName">
  <xsl:element name="DescriptorOrName">
    <xsl:attribute name="MajorTopicYN">
      <xsl:choose>
        <xsl:when test="./@MajorTopicYN">
          <xsl:value-of select="./@MajorTopicYN"/>
        </xsl:when>
        <xsl:otherwise>N</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <xsl:value-of select='.'/>
  </xsl:element>
</xsl:template>

<!-- MeSH Qualifiers and SubHeadings                                -->
<!-- Generate two elements, <QualifierOrSubhead> with an attribute, -->
<!-- MajorTopicYN                                                   -->
<xsl:template match="SubHead|QualifierName">
  <xsl:element name="QualifierOrSubhead">
    <xsl:attribute name="MajorTopicYN">
      <xsl:choose>
        <xsl:when test="./@MajorTopicYN">
          <xsl:value-of select="./@MajorTopicYN"/>
        </xsl:when>
        <xsl:otherwise>N</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <xsl:value-of select='.'/>
  </xsl:element>
</xsl:template>

<!-- Comments and Corrextions                                       -->
<!-- We convert several different elments into a single element     -->
<xsl:template match="CommentsCorrections/*">
  <xsl:element name="CommentOrCorrection">
    <xsl:element name="Type">
      <xsl:value-of select="name()"/>
    </xsl:element>
    <xsl:apply-templates select="RefSource|Note"/>
  </xsl:element>
</xsl:template>

<!-- URL                                                            -->
<!-- The only processing on this element is to convert the type     -->
<!-- attribute (if present) to a single character, 'F' or 'S'       -->
<xsl:template match="URL">
  <xsl:element name="URL">
    <xsl:if test="./@lang">
      <xsl:attribute name="lang">
        <xsl:value-of select="./@lang"/>
      </xsl:attribute>
    </xsl:if>
    <xsl:if test="./@type">
      <xsl:attribute name="type">
        <xsl:choose>
          <xsl:when test="@type='FullText'">F</xsl:when>
          <xsl:when test="@type='fulltext'">F</xsl:when>
          <xsl:when test="@type='Summary'">S</xsl:when>
          <xsl:when test="@type='summary'">S</xsl:when>
          <xsl:otherwise>U</xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
    </xsl:if>
    <xsl:value-of select="."/>
  </xsl:element>
</xsl:template>

<!-- Reject Templates                                               -->
<!-- These templates prevent the "catchall" (see below) template    -->
<!-- from copying elements that we don't care about; this should    -->
<!-- reduce the amount of work that needs to be done by the next    -->
<!-- phase.                                                         -->
<xsl:template match="History"/>

<!-- Utility Templates                                              -->

<!-- "normal" dates                                                 -->
<!-- These are converted to a XML schema format date (yyyy-mm-dd)   -->
<xsl:template name="normal-date">
  <xsl:element name="{name()}">
    <xsl:value-of select = "Year/text()"/>-<xsl:choose>
      <xsl:when test="string-length(Month/text()) = 1">0<xsl:value-of select="Month/text()" /></xsl:when>
      <xsl:otherwise><xsl:value-of select="Month/text()" /></xsl:otherwise>
    </xsl:choose>-<xsl:choose>
      <xsl:when test="string-length(Day/text()) = 1">0<xsl:value-of select="Day/text()" /></xsl:when>
      <xsl:otherwise><xsl:value-of select="Day/text()" /></xsl:otherwise>
    </xsl:choose>
    <!-- <xsl:value-of select="concat(Month/text(), '/', Day/text(), '/', Year/text())"/> -->
  </xsl:element>
</xsl:template>

<xsl:template name="canonical-name">
<xsl:value-of select="LastName/text()"/>
<xsl:choose>
<xsl:when test="ForeName">, <xsl:value-of select="ForeName/text()"/></xsl:when>
<xsl:otherwise>
<xsl:choose>
<xsl:when test="FirstName">, <xsl:value-of select="FirstName/text()"/><xsl:if test="MiddleName"> <xsl:value-of select="MiddleName/text()"/></xsl:if></xsl:when>
<xsl:otherwise>
<xsl:if test="Initials">, <xsl:value-of select="Initials/text()"/></xsl:if>
</xsl:otherwise>
</xsl:choose>
</xsl:otherwise>
</xsl:choose>
<xsl:if test="Affiliation"> (<xsl:value-of select="Affiliation/text()"/>)</xsl:if>
</xsl:template>

<xsl:template name="name-list">
  <xsl:param name="listName"/>
  <xsl:element name="{$listName}">
    <xsl:for-each select="child::*">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:call-template name="canonical-name"/>
    </xsl:for-each>
  </xsl:element>
</xsl:template>

<xsl:template name="semicolon-list">
  <xsl:param name="listName"/>
  <xsl:element name="{$listName}">
    <xsl:for-each select="child::*">
      <xsl:if test="position()>1">; </xsl:if>
      <xsl:value-of select="."/>
    </xsl:for-each>
  </xsl:element>
</xsl:template>

<!-- MedlineCitation element                                                -->
<!--   Per the DTD, the Owner attribute has a default value of NLM          -->
<!--   Apply the default value to the attribute Owner if Owner is missing   -->
<xsl:template match="MedlineCitation">
  <xsl:element name="MedlineCitation">
    <xsl:attribute name="Owner">
      <xsl:choose>
        <xsl:when test="@Owner">
          <xsl:value-of select="@Owner"/>
        </xsl:when>
        <xsl:otherwise>NLM</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>

    <xsl:apply-templates select="@* | node()"/>

  </xsl:element>
</xsl:template>


<!-- "Catchall" template                                            -->
<!-- This template causes any element, not otherwise processed      -->
<!-- to be copied to the output                                     -->
<xsl:template match="*">
  <xsl:element name="{name()}">
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates select="*|text()"/>
  </xsl:element>
</xsl:template>

<xsl:template match="@*">
  <xsl:attribute name="{name()}">
    <xsl:value-of select="."/>
  </xsl:attribute>
</xsl:template>

</xsl:stylesheet>
